import java.util.*;



public class Graph {

    private Map nodes; // Name -> Node

    /**
     * creates an empty graph
     */
    public Graph() {
        this.nodes = new HashMap();
    }

    /**
     * adds the node with given name to the graph
     */
    public void addNode(String name) throws DuplicateNodeException {
        if (!this.nodes.containsKey(name)) {
            this.nodes.put(name,new Node(name));
        } else {
            throw new DuplicateNodeException(
               "Node "+name+" already in the graph!");
        }
    }

    /*
     * looks up the node for the string
     */
    private Node getNode(String name) throws NodeNotPresentException {
        Node n = (Node) this.nodes.get(name);
        if (n == null) {
            throw new NodeNotPresentException(
              "The node "+name+" does not exist in the graph");
        }
        return n;
    }


    /**
     * adds to the graph an edge from "from" to "to"
     * @throw Exception if one of the nodes is not in the graph or
     *        if the edge is already in the graph
     */
    public void addEdge(String from, String to) throws GraphException {
        Node start = getNode(from); // NodeNotPresentE
        Node end =   getNode(to);   // NodeNotPresentE
        start.addEdgeTo(end);       // DuplicateEdgeE
    }


    /**
     * searches for a path from "from" to "to"
     * @return null, if there is no path;
     *         a string-representation of the path otherwise
     * @throw Exception if one of the given nodes is not in the graph
     */
    public String findPath(String from, String to)
                               throws NodeNotPresentException {
        Node start = getNode(from);
        Node end = getNode(to);

        // reset visit labels
        Iterator i = nodes.values().iterator();
        while (i.hasNext()) {
            Node n = (Node) i.next();
            n.resetVisit();
        }

        // start search
        return search(start,end);
    }


    /*
     * searches for a path from node to goal
     * uses visit-marks for termination
     */
    private static String search(Node node, Node goal) {
        // reached goal?
        if (node == goal) {
            return (goal.getName());
        }

        // already visited? Then do not continue search here
        if (node.isVisited()) {
            return null;
        }

        // mark node visited
        node.visit();

        // look in all ways
        Iterator i = node.getSuccIterator();
        while (i.hasNext()) {
            Node succ = (Node) i.next();
            String path = search(succ,goal);
            if (path != null) {
                return (node.getName() + " -> "+path);
            }
        }

        // no way from this node
        return null;

    }


    /**
     * dot-representation of the graph;
     * use package graphviz (dot/dotty) to display the graph
     */
    public String toString() {
        String s = "digraph g {\n";
        Iterator i = this.nodes.values().iterator();
        while (i.hasNext()) {
            s += i.next();
        }
        return s+"}";
    }




    ////////////////////////////////////////////////////////////////////////////
    ////////////////////////////////////////////////////////////////////////////
    ////////////////////////////////////////////////////////////////////////////
    ////////////////////////////////////////////////////////////////////////////



    /*
     * a subclass for the nodes of the graph
     */
    private class Node {

        /*
         * a node has a name, a visit-flag and a set of successors
         */
        private String name;
        private boolean visited;
        private HashSet succs; // of nodes


        /*
         * construct node with corresponding name
         */
        private Node(String name) {
            this.name = name;
            this.visited = false;
            this.succs = new HashSet();
        }

        /*
         * add edge to other node
         * throws exception if edge already present
         */
        private void addEdgeTo(Node n) throws DuplicateEdgeException {
            if (!this.succs.add(n)) {
                throw new DuplicateEdgeException(
                    "Edge from "+this.name+" to "+
                    n.name+" already in the graph!");
            }
        }

        /*
         * gives iterator over successors
         */
        private Iterator getSuccIterator() {
            return this.succs.iterator();
        }

        /*
         * returns the name of the node
         */
        private String getName() {
            return this.name;
        }

        /*
         * resets the visit flag to false
         */
        private void resetVisit() {
            this.visited = false;
        }

        /*
         * marks the node visited
         */
        private void visit() {
            this.visited = true;
        }

        /*
         * checks whether node was visited
         */
        private boolean isVisited() {
            return this.visited;
        }

        /*
         * output for dotty format
         */
        public String toString() {
            String s = "";
            Iterator i = this.succs.iterator();
            while (i.hasNext()) {
                Node n = (Node) i.next();
                s += this.name + " -> "+n.name+";\n";
            }
            return s;
        }

    }

    //////////////////////////////////////////////////
    //////////////////////////////////////////////////
    //////////////////////////////////////////////////
    //////////////////////////////////////////////////
    //////////////////////////////////////////////////




    /*
     * a subclass for the nodes of the graph
     * without exceptions
     */
    private class NodeI {

        /*
         * a node has a name, a visit-flag and a set of successors
         */
        private String name;
        private HashSet succs; // of nodes


        /*
         * construct node with corresponding name
         */
        private NodeI(String name) {
            this.name = name;
            this.succs = new HashSet();
        }

        /*
         * add edge to other node
         * throws exception if edge already present
         */
        private void addEdgeTo(Node n) {
            this.succs.add(n);
        }

        /*
         * gives iterator over successors
         */
        private Iterator getSuccIterator() {
            return this.succs.iterator();
        }

        /*
         * returns the name of the node
         */
        private String getName() {
            return this.name;
        }


        /*
         * output for dotty format
         */
        public String toString() {
            String s = "";
            Iterator i = this.succs.iterator();
            while (i.hasNext()) {
                Node n = (Node) i.next();
                s += this.name + " -> "+n.name+";\n";
            }
            return s;
        }

    }

    //////////////////////////////////////////////////
    //////////////////////////////////////////////////
    //////////////////////////////////////////////////
    //////////////////////////////////////////////////
    //////////////////////////////////////////////////


    /*
     * Alternative implementation of Node without collection classes
     * possible, but there is work to do ...
     */
    private class NodeII {

        private String name;
        private NodeList succs;

        // ...

    }

    private class NodeList {

        private ListElem head;

        // ...

        private class ListElem {

            private NodeII elem;
            private ListElem next;

            // ...

        }


    }



}

